@@ -1,11 +1,11 @@
-;;; eudc.el --- Emacs Unified Directory Client - LDAP Support
+;;; eudc-ldap.el --- Emacs Unified Directory Client - LDAP Backend
 
 ;; Copyright (C) 1998 Free Software Foundation, Inc.
 
 ;; Author: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
 ;; Maintainer: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
 ;; Created: Feb 1998
-;; Version: $Revision: 1.8 $
+;; Version: $Revision: 1.10 $
 ;; Keywords: help
 
 ;; This file is part of XEmacs
@@ -42,46 +42,8 @@
 (require 'ldap)
 
 
-;;{{{      Package customization variables
-
-(defgroup eudc-ldap nil 
-  "Emacs Unified Directory Client - LDAP Support"
-  :group 'comm)
-
-(defcustom eudc-ldap-bbdb-conversion-alist
-  '((name . cn)
-    (net . mail)
-    (address . (eudc-bbdbify-address postaladdress "Address"))
-    (phone . ((eudc-bbdbify-phone telephonenumber "Phone"))))
-  "*A mapping from BBDB to LDAP attributes.
-This is a list of cons cells (BBDB-FIELD . SPEC-OR-LIST) where
-BBDB-FIELD is the name of a field that must be defined in your BBDB
-environment (standard field names are `name', `company', `net', `phone',
-`address' and `notes').  SPEC-OR-LIST is either a single SPEC or a list
-of SPECs.  Lists of specs are valid only for the `phone' and `address'
-BBDB fields.  SPECs are sexps which are evaluated:
-  a string evaluates to itself,
-  a symbol evaluates to the symbol value.  Symbols naming LDAP attributes
-    present in the record evaluate to the value of the field in the record,
-  a form is evaluated as a function.  The argument list may contain LDAP 
-    field names which eval to the corresponding values in the
-    record.  The form evaluation should return something appropriate for
-    the particular BBDB-FIELD (see `bbdb-create-internal').
-    `eudc-bbdbify-phone' and `eudc-bbdbify-address' are provided as convenience
-    functions to parse phones and addresses."
-  :tag "BBDB to LDAP Attribute Names Mapping"
-  :type '(repeat (cons :tag "Field Name"
-		       (symbol :tag "BBDB Field")
-		       (sexp :tag "Conversion Spec")))
-  :group 'eudc-ldap)
-;;}}}
-
 ;;{{{      Internal cooking
 
-;; Whether eudc-inline-query-format should be cleared on switching protocols
-;; from LDAP
-(defvar eudc-ldap-clear-inline-query-format-on-exit nil)
-
 (defvar eudc-ldap-attributes-translation-alist
   '((name . sn)
     (firstname . givenname)
@@ -94,8 +56,7 @@ BBDB fields.  SPECs are sexps which are evaluated:
     (eudc-list-attributes-function . eudc-ldap-get-field-list)
     (eudc-protocol-attributes-translation-alist . eudc-ldap-attributes-translation-alist)
     (eudc-bbdb-conversion-alist . eudc-ldap-bbdb-conversion-alist)
-    (eudc-protocol-switch-init-function . eudc-ldap-protocol-switch-init)
-    (eudc-protocol-switch-exit-function . eudc-ldap-protocol-switch-exit))
+    (eudc-protocol-has-default-query-attributes . nil))
   "LDAP protocol specific values of EUDC variables.
 This should be an alist of the form (EUDC-VAR . LDAP-VAR) where
 EUDC-VAR is the name of a EUDC variable and LDAP-VAR is 
@@ -163,20 +124,6 @@ attribute names are returned. Default to `person'"
 					    (format "(%s=%s)" (car item) (cdr item)))
 					 query))))
 
-;; Set eudc-inline-query-format for LDAP since LDAP does not
-;; have default query attributes and hence won't accept plain
-;; string queries without an associated attribute name
-(defun eudc-ldap-set-inline-query-format-maybe ()
-  (when (null eudc-inline-query-format)
-    (setq eudc-inline-query-format '(name))
-    (setq eudc-ldap-clear-inline-query-format-on-exit t)))
-
-(defun eudc-ldap-protocol-switch-init ()
-  (eudc-ldap-set-inline-query-format-maybe))
-
-(defun eudc-ldap-protocol-switch-exit ()
-  (if eudc-ldap-clear-inline-query-format-on-exit
-      (setq eudc-inline-query-format nil)))
 
 ;;}}}        
 
