@@ -1,11 +1,11 @@
-;;; eudc.el --- Emacs Unified Directory Client - CCSO PH/QI Support
+;;; eudc-ph.el --- Emacs Unified Directory Client - CCSO PH/QI Backend
 
 ;; Copyright (C) 1998 Free Software Foundation, Inc.
 
 ;; Author: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
 ;; Maintainer: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
 ;; Created: Feb 1998
-;; Version: $Revision: 1.4 $
+;; Version: $Revision: 1.6 $
 ;; Keywords: help
 
 ;; This file is part of XEmacs
@@ -40,48 +40,14 @@
 
 (require 'eudc)
 
-;;{{{      Package customization variables
-
-(defgroup eudc-ph nil 
-  "Emacs Unified Directory Client - CCSO PH/QI Support"
-  :group 'comm)
-
-(defcustom eudc-ph-bbdb-conversion-alist
-  '((name . name)
-    (net . email)
-    (address . (eudc-bbdbify-address address "Address"))
-    (phone . ((eudc-bbdbify-phone phone "Phone")
-	      (eudc-bbdbify-phone office_phone "Office Phone"))))
-  "*A mapping from BBDB to PH/QI fields.
-This is a list of cons cells (BBDB-FIELD . SPEC-OR-LIST) where
-BBDB-FIELD is the name of a field that must be defined in your BBDB
-environment (standard field names are `name', `company', `net', `phone',
-`address' and `notes').  SPEC-OR-LIST is either a single SPEC or a list
-of SPECs.  Lists of specs are valid only for the `phone' and `address'
-BBDB fields.  SPECs are sexps which are evaluated:
-  a string evaluates to itself,
-  a symbol evaluates to the symbol value.  Symbols naming PH/QI fields
-    present in the record evaluate to the value of the field in the record,
-  a form is evaluated as a function.  The argument list may contain PH/QI 
-    field names which eval to the corresponding values in the
-    record.  The form evaluation should return something appropriate for
-    the particular BBDB-FIELD (see `bbdb-create-internal').
-    `eudc-bbdbify-phone' and `eudc-bbdbify-address' are provided as convenience
-    functions to parse phones and addresses."
-  :tag "BBDB to CCSO Field Name Mapping"
-  :type '(repeat (cons :tag "Field Name"
-		       (symbol :tag "BBDB Field")
-		       (sexp :tag "Conversion Spec")))
-  :group 'eudc-ph)
-
-;;}}}
 
 ;;{{{      Internal cooking
 
 (defconst eudc-ph-protocol-locals 
   '((cons eudc-bbdb-conversion-alist . eudc-ph-bbdb-conversion-alist)
     (eudc-query-function . eudc-ph-query-internal)
-    (eudc-list-attributes-function . eudc-ph-get-field-list))
+    (eudc-list-attributes-function . eudc-ph-get-field-list)
+    (eudc-protocol-has-default-query-attributes . t))
   "CCSO PH/QI protocol specific values of EUDC variables.
 This should be an alist of the form (EUDC-VAR . PH-VAR) where
 EUDC-VAR is the name of a EUDC variable and PH-VAR is the 
