@@ -5,7 +5,7 @@
 ;; Author: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
 ;; Maintainer: Oscar Figueiredo <Oscar.Figueiredo@epfl.ch>
 ;; Created: Feb 1998
-;; Version: $Revision: 1.8 $
+;; Version: $Revision: 1.12 $
 ;; Keywords: help
 
 ;; This file is part of XEmacs
@@ -27,7 +27,7 @@
 
 ;;; Commentary:
 ;;    This package provides a common interface to query directory
-;;    servers using different protocols such as LDAP or CCSO PH/QI.
+;;    servers using different protocols such as LDAP, CCSO PH/QI or BBDB.
 ;;    Queries can be made through an interactive form. 
 ;;    Inline query strings in buffers can also be expanded with appropriately
 ;;    formatted query results (especially used to expand email addresses in
@@ -55,172 +55,8 @@
 (autoload 'bbdb-parse-phone-number "bbdb-com")
 (autoload 'bbdb-display-records "bbdb")
 
-;;{{{      Package customization variables
+(require 'eudc-custom-vars)
 
-(defgroup eudc nil 
-  "Emacs Unified Directory Client"
-  :group 'mail
-  :group 'comm)
-
-(defcustom eudc-server nil
-  "*The name or IP address of the directory server.
-A port number may be specified by appending a colon and a
-number to the name of the server."
-  :type  '(string :tag "Server")
-  :group 'eudc)
-
-(defvar eudc-supported-protocols nil
-  "Protocols currently supported by the Unified Directory Client.
-This variable is updated when protocol-specific libraries
-are loaded, *do not change by hand*.")
-
-(defcustom eudc-protocol nil
-  "*The directory protocol to use to query the server.
-Supported protocols are specified by `eudc-supported-protocols'."
-  :type  `(choice :menu-tag "Protocol"
-		  ,@(mapcar (lambda (s) 
-			      (list 'string ':tag (symbol-name s)))
-			    eudc-supported-protocols))
-  :group 'eudc)
-
-
-(defcustom eudc-strict-return-matches t
-  "*If non-nil, ignore entries that do not contain all requested return attributes."
-  :type  'boolean
-  :group 'eudc)
-
-(defcustom eudc-default-return-attributes nil
-  "*A list of the default attributes to extract from directory entries.
-If set to the symbol `all' return all attributes.
-nil means return the default attributes as configured in the server."
-  :type  '(choice :menu-tag "Return Attributes"
-		  (const :menu-tag "Server Defaults (nil)" nil)
-		  (const :menu-tag "All" all)
-		  (repeat :menu-tag "Edit List" (symbol :tag "Field name")))
-  :group 'eudc)
-
-(defcustom eudc-multiple-match-handling-method 'select
-  "*What to do when multiple entries match an inline expansion query.
-Possible values are: 
-`first' (equivalent to nil) which means keep the first match only,
-`select' pop-up a selection buffer,
-`all' expands to all matches,
-`abort' the operation is aborted, an error is signaled."
-  :type  '(choice :menu-tag "Method"
-		  (const :menu-tag "First"  first)
-		  (const :menu-tag "Select" select)
-		  (const :menu-tag "All"    all)
-		  (const :menu-tag "Abort"  abort)
-		  (const :menu-tag "None" nil))
-  :group 'eudc)
-
-(defcustom eudc-duplicate-attribute-handling-method '((email . duplicate))
-  "*A method to handle entries containing duplicate attributes.
-This is either an alist (ATTR . METHOD) or a symbol METHOD.
-The alist form of the variable associates a method to an individual attribute,
-the second form specifies a method applicable to all attributes.
-Available methods are:
-`list' or nil lets the value of the attribute be a list of values,
-`first' keeps the first value and discards the others,
-`concat' concatenates the values into a single multiline string,
-`duplicate' duplicates the entire entry into as many instances as 
-different values."
-  :type '(choice (const :menu-tag "List" list)
-		 (const :menu-tag "First" first)
-		 (const :menu-tag "Concat" concat)
-		 (const :menu-tag "Duplicate" duplicate)
-		 (repeat :menu-tag "Per Attribute Specification"
-			 :tag "Per Attribute Specification"
-			 (cons :tag "Attribute/Method"
-			       :value (nil . list)
-			       (symbol :tag "Attribute name")
-			       (choice :tag "Method"
-				       :menu-tag "Method"
-				       (const :menu-tag "List" list)
-				       (const :menu-tag "First" first)
-				       (const :menu-tag "Concat" concat)
-				       (const :menu-tag "Duplicate" duplicate)))))
-  :group 'eudc
-  )
-
-(defcustom eudc-inline-query-format nil
-  "*Format of an inline expansion query.
-If the inline query string consists of several words, this list specifies 
-how these individual words are mapped onto attribute names.
-If nil all the words will be mapped onto the default server/protocol key."
-  :type  '(repeat (symbol :tag "Attribute name"))
-  :group 'eudc)
-
-(defcustom eudc-expansion-overwrites-query t
-  "*If non nil, expanding a query overwrites the query string."
-  :type  'boolean
-  :group 'eudc)
-
-(defcustom eudc-inline-expansion-format '("%s" email)
-  "*A list specifying the format of the expansion of inline queries.
-This variable controls what `eudc-expand-inline' actually inserts in the buffer.
-First element is a string passed to `format'.  Remaining elements are symbols
-indicating attribute names, the corresponding values are passed
-as additional arguments to `format'."
-  :type  '(list (string :tag "Format String")
-		(repeat :inline t
-			:tag "Attribute names"
-			(symbol :tag "")))
-  :group 'eudc)
-
-(defcustom eudc-query-form-attributes '(name firstname email phone)
-  "*A list of attributes presented in the query form."
-  :tag   "Attributes in Query Forms"
-  :type  '(repeat (symbol :tag "Attribute name"))
-  :group 'eudc)
-
-(defcustom eudc-user-attribute-names-alist '((url . "URL")
-					     (callsign . "HAM Call Sign")
-					     (id . "ID")
-					     (email . "E-Mail")
-					     (firstname . "First Name")
-					     (cn . "Full Name")
-					     (sn . "Surname")
-					     (givenname . "First Name")
-					     (ou . "Unit")
-					     (labeledurl . "URL")
-					     (postaladdress . "Address")
-					     (postalcode . "Postal Code")
-					     (l . "Location")
-					     (c . "Country")
-					     (o . "Organization")
-					     (roomnumber . "Office")
-					     (telephonenumber . "Phone")
-					     (uniqueidentifier . "ID")
-					     (objectclass . "Object Class"))
-  "*Alist of user-defined names for directory attributes.
-These names are used as prompt strings in query/response forms 
-instead of the raw directory attribute names.
-Prompt strings for attributes that are not listed here
-are derived by splitting the attribute name
-at `_' signs and capitalizing the individual words."
-  :tag   "User-defined Names of Directory Attributes"
-  :type  '(repeat (cons :tag "Field"
-			(symbol :tag "Directory attribute")
-			(string :tag "User name")))
-  :group 'eudc)
-
-(defcustom eudc-use-raw-directory-names nil
-  "*If non-nil, use attributes names as defined in the directory.
-Otherwise, directory query/response forms display the user attribute
-names defined in `eudc-user-attribute-names-alist'."
-  :type  'boolean
-  :group 'eudc)
-
-(defcustom eudc-options-file "~/.eudc-options"
-  "*A file where the `servers' hotlist is stored."
-  :type '(file :Tag "File Name:"))
-
-(defcustom eudc-mode-hook nil
-  "*Normal hook run on entry to EUDC mode."
-  :type '(repeat (sexp :tag "Hook")))
-
-;;}}}
 
 
 ;;{{{      Internal cooking
@@ -236,6 +72,7 @@ names defined in `eudc-user-attribute-names-alist'."
 
 (defvar eudc-form-widget-list nil)
 (defvar eudc-mode-map nil)
+(defvar eudc-hotlist-menu nil)
 (defvar eudc-hotlist-mode-map nil)
 (defvar eudc-hotlist-list-beginning nil)
 
@@ -267,6 +104,9 @@ names defined in `eudc-user-attribute-names-alist'."
 ;; another one
 (defvar eudc-protocol-switch-exit-function nil)
 
+;; Whether the protocol supports queries with no specified attribute name
+(defvar eudc-protocol-has-default-query-attributes nil)
+
 ;; Protocol locals
 ;; List of (PROTOCOL (EUDC-VAR . VALUE) (EUDC-VAR .VALUE) ...)
 (defvar eudc-protocol-locals '((default 
@@ -275,7 +115,8 @@ names defined in `eudc-user-attribute-names-alist'."
 				 (eudc-protocol-attributes-translation-alist . nil)
 				 (eudc-bbdb-conversion-alist . nil)
 				 (eudc-protocol-switch-init-function . nil)
-				 (eudc-protocol-switch-exit-function . nil))))
+				 (eudc-protocol-switch-exit-function . nil)
+				 (eudc-protocol-has-default-query-attributes . nil))))
 
 
 ;;; Emacs does not provide that one
@@ -541,7 +382,10 @@ otherwise they are formatted according to `eudc-user-attribute-names-alist'."
       (mapcar (function
 	       (lambda (field)
 		 (let ((method (if (consp eudc-duplicate-attribute-handling-method)
-				   (cdr (assq (car field) eudc-duplicate-attribute-handling-method))
+				   (cdr (assq (or (car (rassq (car field)
+							      (symbol-value eudc-protocol-attributes-translation-alist)))
+						  (car field))
+					      eudc-duplicate-attribute-handling-method))
 				 eudc-duplicate-attribute-handling-method)))
 		   (cond
 		    ((or (null method) (eq 'list method))
@@ -868,10 +712,16 @@ individual inline query words with directory attribute names.
 After querying the server for the given string, the expansion specified by 
 `eudc-inline-expansion-format' is inserted in the buffer at point.
 If REPLACE is non nil, then this expansion replaces the name in the buffer.
-`eudc-expansion-overwrites-query' being non nil inverts the meaning of REPLACE."
+`eudc-expansion-overwrites-query' being non nil inverts the meaning of REPLACE.
+Multiple servers can be tried with the same query until one finds a match, 
+see `eudc-inline-expansion-servers'"
   (interactive)
-  (or eudc-server
-      (call-interactively 'eudc-set-server))
+  (if (memq eudc-inline-expansion-servers 
+	    '(current-server server-then-hotlist))
+      (or eudc-server
+	  (call-interactively 'eudc-set-server))
+    (or eudc-server-hotlist
+	(error "No server in the hotlist")))
   (let* ((end (point))
 	 (beg (save-excursion
 		(if (re-search-backward "[:,][ \t]*" 
@@ -885,19 +735,37 @@ If REPLACE is non nil, then this expansion replaces the name in the buffer.
 	 query
 	 query-alist
 	 (query-format eudc-inline-query-format)
-	 (eudc-inline-expansion-format 
-	  (cons (car eudc-inline-expansion-format)
-		(eudc-translate-attribute-list 
-		 (cdr eudc-inline-expansion-format))))
 	 response
 	 response-string
 	 response-strings
-	 key val cell)
-    
+	 key val cell
+	 (eudc-former-server eudc-server)
+	 (eudc-former-protocol eudc-protocol)
+	 servers
+	 (eudc-inline-query-format (or eudc-inline-query-format
+				       (if (null eudc-protocol-has-default-query-attributes)
+					   '(name)))))
+
+    (setq servers (copy-sequence eudc-server-hotlist))
+    (setq servers
+	  (cond 
+	   ((eq eudc-inline-expansion-servers 'hotlist)
+	    eudc-server-hotlist)
+	   ((eq eudc-inline-expansion-servers 'server-then-hotlist)
+	    (cons (cons eudc-server eudc-protocol)
+		  (delete (cons eudc-server eudc-protocol) servers)))
+	   ((eq eudc-inline-expansion-servers 'current-server)
+	    (list (cons eudc-server eudc-protocol)))
+	   (t
+	    (error "Wrong value for `eudc-inline-expansion-servers': %S"
+		   eudc-inline-expansion-servers))))
+	    
     ;; Prepare the query
-    (if (not query-format)
-	(setq query words)
-      (setq words (split-string words "[ \t]+"))
+    (setq words (split-string words "[ \t]+"))
+    (if (= (length words) 1)
+	(if eudc-protocol-has-default-query-attributes
+	    (setq query (car words))
+	  (setq query (list (cons 'name (car words)))))
       (while (and words query-format)
 	(setq query-alist (cons (cons (car query-format) (car words)) query-alist))
 	(setq words (cdr words)
@@ -915,44 +783,58 @@ If REPLACE is non nil, then this expansion replaces the name in the buffer.
 	  (setq query (cons (car query-alist) query)))
 	(setq query-alist (cdr query-alist))))
 
-    (setq response (eudc-query query (cdr eudc-inline-expansion-format)))
-
-    (if (null response)
-	(error "No match found")
-
-      ;; Process response through eudc-inline-expansion-format
-      (while response
-	(setq response-string (apply 'format 
-				     (car eudc-inline-expansion-format)
-				     (mapcar (function 
-					      (lambda (field)
-						(or (cdr (assq field (car response))) 
-						    "")))
-					     (cdr eudc-inline-expansion-format))))
-	(if (> (length response-string) 0)
-	    (setq response-strings
-	      (cons response-string response-strings)))
-	(setq response (cdr response)))
-
-      (if (or
-	   (and replace (not eudc-expansion-overwrites-query))
-	   (and (not replace) eudc-expansion-overwrites-query))
-	  (delete-region beg end))
-      (cond 
-       ((or (= (length response-strings) 1)
-	    (null eudc-multiple-match-handling-method)
-	    (eq eudc-multiple-match-handling-method 'first))
-	(insert (car response-strings)))
-       ((eq eudc-multiple-match-handling-method 'select)
-	(eudc-select response-strings))
-       ((eq eudc-multiple-match-handling-method 'all)
-	(insert (mapconcat 'identity response-strings ", ")))
-       ((eq eudc-multiple-match-handling-method 'abort)
-	(error "There is more than one match for the query"))
-       ))
-    )
-  )
-
+    (condition-case nil
+	(progn 
+	  (eudc-set-server (caar servers) (cdar servers) t)
+	  (setq servers (cdr servers))
+	  (setq response (eudc-query query (eudc-translate-attribute-list 
+					    (cdr eudc-inline-expansion-format))))
+	  (while (and (null response)
+		      servers)
+	    (eudc-set-server (caar servers) (cdar servers) t)
+	    (setq servers (cdr servers))
+	    (setq response (eudc-query query (eudc-translate-attribute-list
+					      (cdr eudc-inline-expansion-format)))))
+	  
+	  (if (null response)
+	      (error "No match found")
+	    
+	    ;; Process response through eudc-inline-expansion-format
+	    (while response
+	      (setq response-string (apply 'format 
+					   (car eudc-inline-expansion-format)
+					   (mapcar (function 
+						    (lambda (field)
+						      (or (cdr (assq field (car response))) 
+							  "")))
+						   (eudc-translate-attribute-list
+						    (cdr eudc-inline-expansion-format)))))
+	      (if (> (length response-string) 0)
+		  (setq response-strings
+			(cons response-string response-strings)))
+	      (setq response (cdr response)))
+	    
+	    (if (or
+		 (and replace (not eudc-expansion-overwrites-query))
+		 (and (not replace) eudc-expansion-overwrites-query))
+		(delete-region beg end))
+	    (cond 
+	     ((or (= (length response-strings) 1)
+		  (null eudc-multiple-match-handling-method)
+		  (eq eudc-multiple-match-handling-method 'first))
+	      (insert (car response-strings)))
+	     ((eq eudc-multiple-match-handling-method 'select)
+	      (eudc-select response-strings))
+	     ((eq eudc-multiple-match-handling-method 'all)
+	      (insert (mapconcat 'identity response-strings ", ")))
+	     ((eq eudc-multiple-match-handling-method 'abort)
+	      (error "There is more than one match for the query"))
+	     )))
+      (t
+       (or (and (equal eudc-server eudc-former-server)
+		(equal eudc-protocol eudc-former-protocol))
+	   (eudc-set-server eudc-former-server eudc-former-protocol t))))))
+  
 ;;;###autoload
 (defun eudc-query-form (&optional get-fields-from-server)
   "Display a form to query the directory server.
@@ -1151,6 +1033,7 @@ for the existing fields and displays a corresponding form."
   (interactive)
   (save-excursion
     (set-buffer (find-file-noselect eudc-options-file t))
+    (goto-char (point-min))
     ;; delete the previous setq
     (let ((standard-output (current-buffer))
 	  provide-p
@@ -1162,35 +1045,38 @@ for the existing fields and displays a corresponding form."
 			  (read (current-buffer))
 			(end-of-file (throw 'found nil)))))
 	    (if (listp sexp)
-		(progn
-		  (if (eq (car sexp)  'eudc-set-server)
-		      (progn 
-			(delete-region (save-excursion
-					 (backward-sexp)
-					 (point))
-				       (point))
-			(setq set-server-p t)))
-		  (if (and (eq (car sexp)  'setq)
-			   (eq (eudc-cadr sexp) 'eudc-server-hotlist))
-		      (progn 
-			(delete-region (save-excursion
-					 (backward-sexp)
-					 (point))
-				       (point))
-			(setq set-hotlist-p t)))
-		  (if (and (eq (car sexp)  'provide)
-			   (equal (eudc-cadr sexp) '(quote eudc-options-file)))
-		      (setq provide-p t))
-		  (if (and provide-p
-			   set-hotlist-p
-			   set-server-p)
-		      (throw 'found t)))))))
+		(cond
+		 ((eq (car sexp)  'eudc-set-server)
+		  (delete-region (save-excursion
+				   (backward-sexp)
+				   (point))
+				 (point))
+		  (setq set-server-p t))
+		 ((and (eq (car sexp)  'setq)
+		       (eq (eudc-cadr sexp) 'eudc-server-hotlist))
+		  (delete-region (save-excursion
+				   (backward-sexp)
+				   (point))
+				 (point))
+		  (setq set-hotlist-p t))
+		 ((and (eq (car sexp)  'provide)
+		       (equal (eudc-cadr sexp) '(quote eudc-options-file)))
+		  (setq provide-p t)))
+	      (if (and provide-p
+		       set-hotlist-p
+		       set-server-p)
+		  (throw 'found t))))))
       (if (eq (point-min) (point-max))
-	  (princ ";; This file was automatically generated by eudc.el\n\n"))
-      (if (not provide-p)
-	  (princ "(provide 'eudc-options-file)\n"))
-      (if (not (bolp))
+	  (princ ";; This file was automatically generated by eudc.el.\n\n"))
+      (or provide-p
+	  (save-excursion
+	    (goto-char (point-min))
+	    (forward-sexp)
+	    (backward-sexp)
+	    (princ "(provide 'eudc-options-file)\n")))
+      (or (bolp)
 	  (princ "\n"))
+      (delete-blank-lines)
       (princ "(eudc-set-server ")
       (prin1 eudc-server)
       (princ " '")
@@ -1303,7 +1189,7 @@ This function can only be called from a directory query result buffer."
 
 (defun eudc-menu ()
   (let (command)
-    (append '("Directory")
+    (append '("Directory Search")
 	    (list
 	     (append 
 	      '("Server")
@@ -1325,14 +1211,17 @@ This function can only be called from a directory query result buffer."
 				 (message "Selected directory server is now %s (%s)" 
 					  ,server 
 					  ,proto-name))))
-		    (vector (format "%s (%s)" server proto-name) command t))))
+		    (vector (format "%s (%s)" server proto-name)
+			    command
+			    :style 'radio
+			    :selected `(equal eudc-server ,server)))))
 	       eudc-server-hotlist)
 	      eudc-server-menu))
 	    eudc-tail-menu)))
 
 (defun eudc-install-menu ()
   (cond 
-   (eudc-xemacs-p
+   ((and eudc-xemacs-p (featurep 'menubar))
     (add-submenu '("Tools") (eudc-menu)))
    (eudc-emacs-p
     (easy-menu-define eudc-menu-map eudc-mode-map "Directory Client Menu" (eudc-menu))
@@ -1347,10 +1236,13 @@ This function can only be called from a directory query result buffer."
 ;;; Load time initializations :
 
 ;;; Load the options file
-(if (and (and (locate-library eudc-options-file)
+(if (and (not noninteractive)
+	 (and (locate-library eudc-options-file)
 	      (message ""))		; Remove modeline message
 	 (not (featurep 'eudc-options-file)))
-    (load eudc-options-file))	 
+    (load eudc-options-file))
+  
+	 
 ;;; Install the full menu
 (unless (featurep 'infodock)
   (eudc-install-menu))
@@ -1368,10 +1260,11 @@ This does nothing except loading eudc by autoload side-effect."
 
 ;;;###autoload
 (if (and (string-match "XEmacs" emacs-version)
+	 (featurep 'menubar)
 	 (not (featurep 'eudc))
 	 (not (featurep 'infodock)))
     (add-submenu '("Tools")
-		 '("Directory"
+		 '("Directory Search"
 		   ["Load Hotlist of Servers" eudc-load-eudc t]
 		   ["New Server" eudc-set-server t]
 		   ["---" nil nil]
